from .core import AllStatic, customizable_progress_method
from .fg import FG



class Progress(metaclass=AllStatic):

    def block(progress, total, length=20, filled_color=FG.green, empty_color=FG.red, label="") -> None: customizable_progress_method(progress, total, length, "[", "]", "█", "░", filled_color, empty_color, label=label)

    def arabic(progress, total, length=20, filled_color=FG.green, empty_color=FG.red, label="") -> None:  customizable_progress_method(progress, total, length, "⟨", "⟩", "■", "□", filled_color, empty_color, label=label)

    def professional(progress, total, length=20, filled_color=FG.green, empty_color=FG.red, label="") -> None:  customizable_progress_method(progress, total, length, "|", "|", "▉", "▊", filled_color, empty_color, label=label)

    def minimal(progress, total, length=20, filled_color=FG.green, empty_color=FG.red, label="") -> None:  customizable_progress_method(progress, total, length, "", "", "▪", "▫", filled_color, empty_color, label=label)

    def stars(progress, total, length=20, filled_color=FG.green, empty_color=FG.red, label="") -> None:  customizable_progress_method(progress, total, length, "✨ ", " ✨", "⭐", "☆", filled_color, empty_color, label=label)

    def arrow(progress, total, length=20, filled_color=FG.green, empty_color=FG.red, label="") -> None:  customizable_progress_method(progress, total, length, "> ", " <", "▶", "▷", filled_color, empty_color, label=label)

    def hearts(progress, total, length=20, filled_color=FG.green, empty_color=FG.red, label="") -> None:  customizable_progress_method(progress, total, length, "❤️ ", " ❤️", "♥", "♡", filled_color, empty_color, label=label)








    def gradient(progress, total, length=20, start_color=FG.red, middle_color=FG.yellow, end_color=FG.green, label=""):
        percent = progress / total
        if percent < 0.3:
            color = start_color
        elif percent < 0.7:
            color = middle_color
        else:
            color = end_color
        
        customizable_progress_method(progress, total, length, "[", "]","█", "░",  color, FG.white, label)







__progress_all__ = ["Progress"]


from .core import flatten, logger
from .fg import FG
import re

class Table:
    def __init__(self, 
                title="", justify="left",
                col_separator="|", row_separator="=",
                title_color=FG.white, col_color=FG.white,
                row_color=FG.white, col_separator_color=FG.white,
                row_separator_color=FG.white, cell_padding=0, row_height=1,
                # Borders (user can customize)
                top_left="+", top_right="+",
                bottom_left="+", bottom_right="+",
                horizontal="-", vertical="|",
                cross="+", left_cross="+",
                right_cross="+", top_cross="+",
                bottom_cross="+"):
        self.title = title
        self.justify = justify
        self.columns = []
        self.rows = []
        self.col_widths = []
        self.total_width = 0
        self.cell_padding = cell_padding
        self.row_height = max(1, row_height)

        # Colors
        self.title_color = title_color
        self.col_color = col_color
        self.row_color = row_color
        self.col_separator_color = col_separator_color
        self.row_separator_color = row_separator_color

        # Separator characters
        self.col_separator = col_separator
        self.row_separator = row_separator

        # Borders
        self.tl = top_left
        self.tr = top_right
        self.bl = bottom_left
        self.br = bottom_right
        self.h  = horizontal
        self.v  = vertical
        self.cross = cross
        self.lc = left_cross
        self.rc = right_cross
        self.tc = top_cross
        self.bc = bottom_cross

    def __str__(self):
        return f"<Table: {len(self.columns)} cols, {len(self.rows)} rows>"

    # --------------------------
    # Core Utilities
    # --------------------------
    def strip_ansi_codes(self, text) -> str:
        ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
        return ansi_escape.sub('', str(text))

    def calcul_col_width(self) -> None:
        if not self.columns and not self.rows:
            return
        num_cols = max(len(self.columns), len(self.rows[0]) if self.rows else 0)
        self.col_widths = [0] * num_cols
        
        # Columns width
        for j in range(len(self.columns)):
            length = len(self.strip_ansi_codes(self.columns[j]))
            if length > self.col_widths[j]:
                self.col_widths[j] = length

        # Rows width
        for i in range(len(self.rows)):
            for j in range(len(self.rows[i])):
                length = len(self.strip_ansi_codes(self.rows[i][j]))
                if length > self.col_widths[j]:
                    self.col_widths[j] = length

        padding_length = len(self.cell_padding) if isinstance(self.cell_padding, str) else self.cell_padding
        if padding_length > 0:
            self.col_widths = [w + 2 * padding_length for w in self.col_widths]

    def complet_rows(self) -> None:
        for row in self.rows:
            while len(row) < len(self.columns):
                row.append("")

    def complet_columns(self, needed) -> None:
        for _ in range(needed):
            self.columns.append("")

    def add_column(self, *col) -> None:
        flat_col = flatten(col)
        self.columns.extend(flat_col)
        self.complet_rows()
        self.calcul_col_width()

    def add_row(self, *row) -> None:
        flat_row = flatten(row)
        if len(flat_row) > len(self.columns):
            self.complet_columns(len(flat_row) - len(self.columns))
        self.rows.append(flat_row)
        self.complet_rows()
        self.calcul_col_width()

    def reset_total_width(self) -> None:
        self.total_width = 0

    def calcul_total_width(self) -> None:
        if not self.col_widths:
            self.total_width = len(self.title) if self.title else 0
            return
        total_cols_width = sum(self.col_widths)
        separators_count = len(self.col_widths) + 1
        spaces_count = len(self.col_widths) * 2
        self.total_width = total_cols_width + separators_count + spaces_count - 1

    # --------------------------
    # Draw Utilities
    # --------------------------
    def draw_line(self, left, mid, right) -> None:
        line = left
        for i, w in enumerate(self.col_widths):
            line += self.h * (w + 2)
            if i < len(self.col_widths) - 1:
                line += mid
        line += right
        print(self.row_separator_color(line, AP=False))

    # --------------------------
    # Printing Table
    # --------------------------
    def print_table_title(self) -> None:
        if self.title:
            self.calcul_total_width()
            title_clean = self.strip_ansi_codes(self.title)
            spaces = max(0, (self.total_width - len(title_clean)) // 2)
            print(" " * spaces + self.title_color(self.title, AP=False) + "\n")

    def print_table(self) -> None:
        if not self.columns and not self.rows:
            print("[Info: No data to display]")
            return

        self.reset_total_width()
        self.calcul_col_width()
        self.calcul_total_width()
        self.print_table_title()

        # TOP BORDER
        self.draw_line(self.tl, self.tc, self.tr)

        # HEADER
        middle_line = self.row_height // 2
        for line_num in range(self.row_height):
            print(self.col_separator_color(self.v, AP=False), end="")
            for i, col in enumerate(self.columns):
                clean = self.strip_ansi_codes(col)
                if line_num == middle_line:
                    if self.justify == "center":
                        clean = clean.center(self.col_widths[i])
                    elif self.justify == "right":
                        clean = clean.rjust(self.col_widths[i])
                    else:
                        clean = clean.ljust(self.col_widths[i])
                else:
                    clean = " " * self.col_widths[i]
                print(" " + self.col_color(clean, AP=False) + " ", end="")
                print(self.col_separator_color(self.v, AP=False), end="")
            print()

        # HEADER SEPARATOR
        if self.rows:  # فقط إذا كان هناك صفوف
            self.draw_line(self.lc, self.cross, self.rc)

        # ROWS
        for row_idx, row in enumerate(self.rows):
            for line_num in range(self.row_height):
                print(self.col_separator_color(self.v, AP=False), end="")
                for j, cell in enumerate(row):
                    clean = self.strip_ansi_codes(cell)
                    if line_num == middle_line:
                        if self.justify == "center":
                            clean = clean.center(self.col_widths[j])
                        elif self.justify == "right":
                            clean = clean.rjust(self.col_widths[j])
                        else:
                            clean = clean.ljust(self.col_widths[j])
                    else:
                        clean = " " * self.col_widths[j]
                    print(" " + self.row_color(clean, AP=False) + " ", end="")
                    print(self.col_separator_color(self.v, AP=False), end="")
                print()

            # SEPARATOR BETWEEN ROWS (فقط إذا لم يكن آخر صف)
            if row_idx < len(self.rows) - 1:
                self.draw_line(self.lc, self.cross, self.rc)

        # BOTTOM BORDER
        self.draw_line(self.bl, self.bc, self.br)

__tables_all__ = ["Table"]



from os import path
from .core import LoadPalette


ESC = "\x1B"
SEP = ";"
CSI = "["
RESET_STYLE = "[0m"


BASE_DIR = path.dirname(__file__)
ansi16_fg_colors_file_name = path.join(BASE_DIR, "palettes/ansi16_fg_palette.json")
ansi16_bg_colors_file_name = path.join(BASE_DIR, "palettes/ansi16_bg_palette.json")
text_styles_file_name  = path.join(BASE_DIR, "palettes/text_styles.json")
winapi_fg_colors_file_name = path.join(BASE_DIR, "palettes/winapi_fg_palette.txt")
winapi_bg_colors_file_name = path.join(BASE_DIR, "palettes/winapi_bg_palette.txt")


colors = LoadPalette(ansi16_fg_colors_file_name)
BgColors = LoadPalette(ansi16_bg_colors_file_name)
TextStyles = LoadPalette(text_styles_file_name)
WinFGColors = LoadPalette(winapi_fg_colors_file_name)
WinBGColors = LoadPalette(winapi_bg_colors_file_name)


random_color_choices = ["Black", "Red", "Green", "Yellow", "Blue", "Magenta", "Cyan", "White", "Bright Black", "Bright Red", "Bright Green", "Bright Yellow", "Bright Blue", "Bright Magenta", "Bright Cyan", "Bright White"]


__colors_all__ = ["ESC", "SEP", "CSI", "colors", "TextStyles", "RESET_STYLE", "BgColors", "random_color_choices", "WinFGColors", "WinBGColors"]



from .core import AllStatic, mode, handle, kernel32, csbi, logger, pattern
from .colors import ESC, CSI, RESET_STYLE, SEP, WinBGColors, WinFGColors, TextStyles, colors, BgColors, random_color_choices
from time import sleep
from random import choice, randint



class DIRECT(metaclass=AllStatic):

    
    def color(text : str = "", text_style : str = "Normal", color : str = None , bgcolor : str = None, AP=True) -> str:
        try:
            text_str = str(text) if text is not None else ""
            if mode == "ANSI":
                if color == None and bgcolor == None:
                    if AP:
                        print(text_str)
                        return ""
                    else:
                        return text_str
                if bgcolor == None:
                    colored_text = ESC + CSI + str(TextStyles[text_style]) + SEP + colors[color] + "m" + text_str + ESC + RESET_STYLE
                else:
                    colored_text = ESC + CSI + str(TextStyles[text_style]) + colors[color] + SEP + BgColors[bgcolor] + "m" + text_str + ESC + RESET_STYLE
                if AP:
                    print(colored_text)
                    return ""
                else:
                    return colored_text
            elif mode == "WINAPI":
                issuccess = kernel32.SetConsoleTextAttribute(handle, WinFGColors[color] | WinBGColors[bgcolor])
                if issuccess:
                    print(text_str)
                    kernel32.SetConsoleTextAttribute(handle, csbi.wAttributes)
                    return ""
                else:
                    return ""
        except KeyError as e:
            error_msg = "[Invalid key: " + str(e) + "] "
            logger.error(error_msg)
            return ""
        except TypeError as e:
            error_msg = "[Error: " + str(e) + "] "
            logger.error(error_msg)
            return ""
        except Exception as e:
            error_msg = "[Error: " + str(e) + "] "
            logger.error(error_msg)
            return ""



    def rgb(text : str = "", text_style : str = "Normal", R = 0, G = 0, B = 0, BR = 0, BG = 0, BB = 0, AP=True) -> str:
        try:
            text_str = str(text) if text is not None else ""
            if mode == "ANSI":
                colored_text = ESC + CSI + str(TextStyles[text_style]) + SEP + "38;2" + SEP + str(R) + SEP + str(G) + SEP + str(B) + SEP + "48;2" + SEP + str(BR) + SEP + str(BG) + SEP + str(BB) + "m" + text_str + ESC + RESET_STYLE
                if AP:
                    print(colored_text)
                    return ""
                else:
                    return colored_text
            elif mode == "WINAPI":
                colored_text = "[Error: unsupported platform windows < 10]"
                return ""
        except KeyError as e:
            logger.warning("[Invalid key: " + str(e) + "]")
            return ""
        except TypeError as e:
            logger.error("[Error: " + str(e) + "]")
            return ""
        except Exception as e:
            logger.error("[Error: " + str(e) + "]")
            return ""

    def animated_write(text : str = "", delay : float = 0, fg = None, bg = None) -> str:
        try:
            for char in text:
                out = char
                if fg and not bg: out = fg(out, AP=False)
                if bg and not fg: out = bg(out, AP=False)
                if bg and fg:
                    fg_colored = fg(char, AP=False)
                    bg_colored = bg(char, AP=False)
                    fg_code = fg_colored.replace(char, "").replace(ESC + RESET_STYLE, "")
                    bg_code = bg_colored.replace(char, "").replace(ESC + RESET_STYLE, "")
                    out = fg_code + bg_code + char
                if fg is None and bg is None: out = char
                print(out, end="", flush=True)
                sleep(delay)
            print("\x1b[0m")
        except Exception:
            print("\x1b[0m")


    def random_text_colored(text : str = "", text_style : str = "Normal", AP=True) -> str:
        out = "".join(DIRECT.color(c, text_style, choice(random_color_choices)) for c in text)
        if AP:
            print(out)
            return ""
        else:
            return out


    def random_rgb_text(text: str = "", text_style: str = "Normal", AP=True) -> str:
        colored_chars = []
        for char in text:
            R, G, B = [randint(0, 255) for _ in range(3)]
            try:
                colored_char = DIRECT.rgb(char, text_style, R, G, B, AP=False)
                colored_chars.append(colored_char)
            except Exception:
                colored_chars.append(char)
        result = "".join(colored_chars)
        if AP:
            print(result)
            return ""
        else:
            return result
    


    def color256(text : str = "", text_style="Normal", fg_code=0, bg_code=0, AP=True) -> str:
        try:
            if not (0 <= fg_code <= 255):
                raise ValueError("Foreground color code must be 0-255, got " + fg_code)
            if not (0 <= bg_code <= 255):
                raise ValueError(f"Background color code must be 0-255, got " + bg_code)
            if mode == "ANSI":
                int(fg_code)
                int(bg_code)
                text_str = str(text) if text is not None else ""
                colored_text = ESC + CSI + str(TextStyles[text_style]) + SEP + "38;5;" + str(fg_code) + SEP + "48;5;" + str(bg_code) + "m" + text_str + ESC + RESET_STYLE
                if AP:
                    print(colored_text)
                    return ""
                else:
                    return colored_text
            else:
                logger.error("[Unsupported Platform windows < 10]")
        except (ValueError, TypeError):
            raise RuntimeError("Unsupported Code: " + str(fg_code) + ", " + str(bg_code))
        except KeyError as e:
            return ESC + CSI + str(TextStyles["Normal"]) + ESC + "38;5" + str(fg_code) + SEP + "48;5;" + str(bg_code) + "m" + text_str + ESC + RESET_STYLE
        except Exception as e:
            logger.error("Error in color256: " + str(e))
            if AP:
                print(text_str)
                return ""
            else:
                return text_str

    def sprint(text, AP=True) -> str:
        finale_result = ""
        last_end = 0
        result = []
        for m in pattern.finditer(text):
            start, end = m.span()
            result.append(text[last_end:start])
            color, content = m.group("tag").capitalize(), m.group("body")
            result.append(f"\x1b[{colors.get(color, '37')}m{content}\x1b[0m")
            last_end = end
        result.append(text[last_end:])
        for i in range(len(result)):
            finale_result += result[i]
        if AP:
            print(finale_result)
            return ""
        else:
            return finale_result

__direct_all__ = ["DIRECT"]


from .core import GetSystemInfo, init, AllStatic, handle, kernel32, mode, csbi, LoadPalette, flatten, customizable_progress_method, mouvement_creator_method, logger, __core_all__

AllInfo = GetSystemInfo()
init()

from .fg import __fg_all__, FG
from .bg import __bg_all__, BG
from .direct import __direct_all__, DIRECT
from .colors import ESC, SEP, CSI, colors, TextStyles, RESET_STYLE, BgColors, random_color_choices, WinFGColors, WinBGColors, __colors_all__
from .progress import __progress_all__, Progress
from .cursor import __cursor_all__, CURSOR
from .tables import __tables_all__, Table


__all__ = __colors_all__ + __core_all__ + __bg_all__ + __fg_all__ + __direct_all__ + __progress_all__ + __cursor_all__ + __tables_all__ + ["AllInfo"]
__name__ = "nscolors"
__author__ = "NullSpecter404"
__version__ = "1.0.0"


from .core import AllStatic, handle, kernel32, mode, csbi, logger
from .colors import ESC, CSI, RESET_STYLE, WinFGColors


class FG(metaclass=AllStatic):

    def _print_fg(text, ansicode, wincolor, AP : bool = True) -> str:
        try:
            text_str = str(text) if text is not None else ""
            if mode == "ANSI":
                colored_text = ESC + CSI + ansicode + text_str + ESC + RESET_STYLE
                if AP:
                    print(colored_text)
                    return ""
                else:
                    return colored_text
            elif mode == "WINAPI":
                if handle is None or kernel32 is None:
                    raise RuntimeError("[Error: system not ready]")
                issuccess = kernel32.SetConsoleTextAttribute(handle, WinFGColors[wincolor])
                if issuccess:
                    print(text_str, end="", flush=True)
                    kernel32.SetConsoleTextAttribute(handle, csbi.wAttributes)
                else:
                    raise RuntimeError("[Unexpected error]")
        except (ValueError, TypeError) as e:
            error_msg = "[Value Error: " + str(e) + "]"
            logger.error(error_msg)
            return ""
            
        except (OSError, WindowsError) as e:
            error_msg = "[System Error: " + str(e) + "]"
            logger.error(error_msg)
            return ""
            
        except KeyError as e:
            error_msg = "[Color '" + str(wincolor) + "' not found]"
            logger.error(error_msg)
            return ""
            
        except Exception as e:
            error_msg = "[Unexpected: " + str(type(e).__name__) + "]"
            logger.error(error_msg)
            return ""


    def black(text : str, AP : bool = True) -> str: return FG._print_fg(text, "0;30m", "Black", AP)
    def red(text : str, AP : bool = True) -> str: return FG._print_fg(text, "0;31m", "Red", AP)
    def green(text : str, AP : bool = True) -> str: return FG._print_fg(text, "0;32m", "Green", AP)
    def yellow(text : str, AP : bool = True) -> str: return FG._print_fg(text, "0;33m", "Yellow", AP)
    def blue(text : str, AP : bool = True) -> str: return FG._print_fg(text, "0;34m", "Blue", AP)
    def magenta(text : str, AP : bool = True) -> str: return FG._print_fg(text, "0;35m", "Magenta", AP)
    def cyan(text : str, AP : bool = True) -> str: return FG._print_fg(text, "0;36m", "Cyan", AP)
    def white(text : str, AP : bool = True) -> str: return FG._print_fg(text, "0;37m", "White", AP)
    def intense_black(text: str, AP : bool = True) -> str: return FG._print_fg(text, "0;90m", "Intense Black", AP)
    def intense_red(text: str, AP : bool = True) -> str: return FG._print_fg(text, "0;91m", "Intense Red", AP)
    def intense_green(text: str, AP : bool = True) -> str: return FG._print_fg(text, "0;92m", "Intense Green", AP)
    def intense_yellow(text: str, AP : bool = True) -> str: return FG._print_fg(text, "0;93m", "Intense Yellow", AP)
    def intense_blue(text: str, AP : bool = True) -> str: return FG._print_fg(text, "0;94m", "Intense Blue", AP)
    def intense_magenta(text: str, AP : bool = True) -> str: return FG._print_fg(text, "0;95m", "Intense Magenta", AP)
    def intense_cyan(text: str, AP : bool = True) -> str: return FG._print_fg(text, "0;96m", "Intense Cyan", AP)
    def intense_white(text: str, AP : bool = True) -> str: return FG._print_fg(text, "0;97m", "Intense White", AP)


__fg_all__ = ["FG"]


from .core import AllStatic, logger
from .core import mouvement_creator_method

class CURSOR(metaclass=AllStatic):
    def up(step=1, text: str = "") -> None: 
        mouvement_creator_method(step, "A", text)
    
    def down(step=1, text: str = "") -> None: 
        mouvement_creator_method(step, "B", text)
    
    def right(step=1, text: str = "") -> None: 
        mouvement_creator_method(step, "C", text)
    
    def left(step=1, text: str = "") -> None: 
        mouvement_creator_method(step, "D", text)
    
    def save_position() -> None: 
        print("\x1b[s", end="", flush=True)
    
    def restore_position() -> None: 
        print("\x1b[u", end="", flush=True)
    
    def clear_screen() -> None: 
        print("\x1b[2J", end="", flush=True)
    
    def home() -> None: 
        print("\x1b[H", end="", flush=True)
    
    def goto(row=0, col=0, text: str = "") -> None:

        try:
            row_int = max(0, int(row))
            col_int = max(0, int(col))
            mouvement_creator_method(0, "", text, x=col_int, y=row_int)
        except (ValueError, TypeError):
            mouvement_creator_method(0, "", text, x=0, y=0)
        except Exception as e:
            logger.error("[Goto Error: " + str(e) + "]")
    
    def hide_cursor() -> None:
        try:
            print("\x1b[?25l", end="", flush=True)
        except Exception as e:
            logger.error("[Hide Cursor Error: " + str(e) + "]")
    
    def show_cursor() -> None:
        try:
            print("\x1b[?25h", end="", flush=True)
        except Exception as e:
            logger.error("[Show Cursor Error: " + str(e) + "]")
    
    def clear_line() -> None:
        try:
            print("\x1b[2K", end="", flush=True)
        except Exception as e:
            logger.error("[Clear Line Error: " + str(e) + "]")

__cursor_all__ = ["CURSOR"]


from platform import system, version, release
from ast import literal_eval
from json import load
import logging
import re

handle = None
kernel32 = None
csbi = None


logger = logging.getLogger("nscolors")

file_mode = None
pattern = re.compile(r"<(?P<tag>\w+)>(?P<body>.*?)</(?P=tag)>", re.DOTALL)

try:
    if system().lower() == "windows":
        from ctypes import Structure, c_short, c_ushort, byref, windll
        class COORD(Structure):
            _fields_ = [("X", c_short), ("Y", c_short)]

        class SMALL_RECT(Structure):
            _fields_ = [("Left", c_short), ("Top", c_short),
                        ("Right", c_short), ("Bottom", c_short)]

        class CONSOLE_SCREEN_BUFFER_INFO(Structure):
            _fields_ = [
                ("dwSize", COORD),
                ("dwCursorPosition", COORD),
                ("wAttributes", c_ushort),
                ("srWindow", SMALL_RECT),
                ("dwMaximumWindowSize", COORD)
            ]

        kernel32 = windll.kernel32
        STD_OUTPUT_HANDLE = -11
        handle = kernel32.GetStdHandle(STD_OUTPUT_HANDLE)
        csbi = CONSOLE_SCREEN_BUFFER_INFO()

        if kernel32.GetConsoleScreenBufferInfo(handle, byref(csbi)):
            pass
        else:
            raise RuntimeError("Failed to get console info")
except Exception as e:
    raise RuntimeError("[Error: " + str(e) + "]")


class AllStatic(type):
    def __new__(cls, name, bases, dct):
        new_dict = {}
        for k, v in dct.items():
            if k.startswith("__") and k.endswith("__"):
                new_dict[k] = v
                continue
            if callable(v):
                new_dict[k] = staticmethod(v)
            elif isinstance(v, type):
                new_dict[k] = AllStatic(v.__name__, v.__bases__, dict(v.__dict__))
            else:
                new_dict[k] = v
        return super().__new__(cls, name, bases, new_dict)



mode = None
AllInfo = None

def GetSystemInfo() -> tuple:
    global mode, AllInfo
    info = {
        "system": system(),
        "release": release(),
        "version": version(),
    }
    
    if info["system"].lower() == "windows":
        rel = info["release"]
        try:
            rel_num = int(rel)
        except ValueError:
            mapping = {"XP": 5, "Vista": 6, "7": 7, "8": 8, "8.1": 8, "10": 10, "11": 11}
            rel_num = mapping.get(rel, 10)
        
        if rel_num >= 10:
            mode = "ANSI"
        else:
            mode = "WINAPI"
    else:
        mode = "ANSI"
    
    AllInfo = (mode, info)
    return AllInfo

def WinSetup() -> None:
    try:
        global kernel32, handle
        if AllInfo is None:
            GetSystemInfo()

        mode, info = AllInfo
        if mode == "ANSI" and info["system"].lower() == "windows":
            kernel32 = windll.kernel32
            handle = kernel32.GetStdHandle(-11)
        elif mode == "WINAPI":
            kernel32 = windll.kernel32
            handle = kernel32.GetStdHandle(-11)
    except Exception as e:
        logger.warning("Error: " + str(e) + "]")

def init() -> None:
    if AllInfo is None:
        GetSystemInfo()
    if kernel32 is None or handle is None:
        WinSetup()
    
    mode, info = AllInfo
    if mode == "ANSI" and info["system"].lower() == "windows":
        kernel32.SetConsoleMode(handle, 0x0004 | 0x0001 | 0x0002)


def LoadPalette(file_name) -> dict:
    try:
        global file_mode
        if file_name.endswith(".json"):
            file_mode = "json"
        elif file_name.endswith(".txt"):
            file_mode = "txt"
        with open(file_name, "r", encoding="utf-8") as file:
            if file_mode == "json":
                data = load(file)
                return data
            elif file_mode == "txt":
                data = literal_eval(file.read())
                ToHexDict = {}
                for key, value in data.items():
                    ToHexDict[key] = hex(value)
                return ToHexDict
    except Exception as e:
        raise RuntimeError("Error: " + str(e) + "]")


def flatten(items) -> list:
    result = []
    for item in items:
        if isinstance(item, (list, tuple)):
            result.extend(flatten(item))
        else:
            result.append(str(item))
    return result


def customizable_progress_method(progress, total, length ,start_char, end_char, filled_char, empty_char, filled_color, empty_color, label="") -> None:
    try:
        points = "" if label == "" else " : "
        if total == 0:
            percent = 0
        else:
            percent = progress / total
        percent = max(0.0, min(percent, 1.0))
        filled = int(length*percent)
        bar = filled_color(filled_char, AP=False) * filled + empty_color(empty_char, AP=False) * (length - filled)
        print(label + points + start_char + bar + end_char + " " + str(int(percent*100)) + "%",end="\r", flush=True)
        if percent >= 1.0:
            print()
    except Exception as e:
        error_msg = "[Progress Error: " + str(e) + "]"
        logger.error(error_msg)


def mouvement_creator_method(step, direction, text, x=None, y=None) -> None:
    global csbi
    try:
        step_int = int(step)
        step_int = max(1, step_int)
    except (ValueError, TypeError):
        logger.warning("[Unsupported Value: " + str(step) + "]")
        return

    try:
        xi = yi = None
        if x is not None and y is not None:
            try:
                xi = int(x)
                yi = int(y)
            except (ValueError, TypeError):
                logger.warning("Invalid cursor coordinates: x=" + str(x) + ", y=" + str(y) + ". Falling back to relative movement.")
                xi = yi = None


        if xi is not None and yi is not None:
            if mode == "ANSI":
                row = max(0, yi) + 1
                col = max(0, xi) + 1
                try:
                    print("\x1b[" + str(row) + ";" + str(col) + "H", end="", flush=True)
                except Exception:

                    pass

                try:
                    if 'kernel32' in globals() and kernel32 is not None and 'SetConsoleCursorPosition' in dir(kernel32):
                        try:
                            coord = COORD(xi, yi)
                            kernel32.SetConsoleCursorPosition(handle, coord)
                        except Exception:
                            pass
                except Exception:
                    pass

                if text:
                    print(text, end="", flush=True)
                return

            elif mode == "WINAPI":
                try:
                    try:
                        kernel32.GetConsoleScreenBufferInfo(handle, byref(csbi))
                    except Exception:
                        pass

                    try:
                        max_x = int(csbi.dwSize.X) - 1
                        max_y = int(csbi.dwSize.Y) - 1
                    except Exception:
                        max_x = None
                        max_y = None

                    if max_x is not None and max_y is not None:
                        if xi < 0 or yi < 0 or xi > max_x or yi > max_y:
                            logger.warning("Cursor coords out of range: (" + str(xi) + "," + str(yi) + "). Clamping to buffer bounds.")
                            xi = max(0, min(xi, max_x))
                            yi = max(0, min(yi, max_y))

                    coord = COORD(xi, yi)
                    res = kernel32.SetConsoleCursorPosition(handle, coord)
                    if not res:
                        logger.warning("SetConsoleCursorPosition failed for (" + str(xi) + "," + str(yi) + ").")
                except Exception as e:
                    logger.warning("WINAPI cursor positioning failed: " + str(e))

                if text:
                    print(text, end="", flush=True)
                return

        if mode == "ANSI":
            try:
                print("\x1b[" + str(step_int) + str(direction), end="", flush=True)
            except Exception:
                pass
            if text:
                print(text, end="", flush=True)
            return

        elif mode == "WINAPI":
            try:
                try:
                    kernel32.GetConsoleScreenBufferInfo(handle, byref(csbi))
                except Exception:
                    pass


                try:
                    cur = csbi.dwCursorPosition
                    cur_x = int(cur.X)
                    cur_y = int(cur.Y)
                except Exception:

                    cur_x = 0
                    cur_y = 0

                new_x = cur_x
                new_y = cur_y
                d = str(direction)
                if d == "A":
                    new_y = cur_y - step_int
                elif d == "B":
                    new_y = cur_y + step_int
                elif d == "C":
                    new_x = cur_x + step_int
                elif d == "D":
                    new_x = cur_x - step_int
                else:
                    pass

                try:
                    max_x = int(csbi.dwSize.X) - 1
                    max_y = int(csbi.dwSize.Y) - 1
                except Exception:
                    max_x = None
                    max_y = None

                if max_x is not None and max_y is not None:
                    new_x = max(0, min(new_x, max_x))
                    new_y = max(0, min(new_y, max_y))

                coord = COORD(new_x, new_y)
                res = kernel32.SetConsoleCursorPosition(handle, coord)
                if not res:
                    logger.warning("SetConsoleCursorPosition failed for (" + str(new_x) + "," + str(new_y) + ").")
            except Exception as e:
                logger.error("[Cursor Error: " + str(e) + "]")

            if text:
                print(text, end="", flush=True)
            return

    except Exception as e:
        error_msg = "[Cursor Error: " + str(e) + "]"
        logger.error(error_msg)





__core_all__ = ["AllStatic", "handle", "kernel32", "mode", "AllInfo", "GetSystemInfo", "init", "csbi", "LoadPalette", "flatten", "customizable_progress_method", "mouvement_creator_method", "logger"]



